import logging
import requests
import json

from typing import List, Tuple, Optional
from requests.compat import urljoin

from ppa import ui, appliance
from ppa_appliance import vaults

from . import exceptions
from .records import Secret, SecretData

logger = logging.getLogger(__name__)

driver_name = "azure-key-vault"
api_version = "7.1"


def _proxies():
    return {"https": appliance.api_proxy(), "http": appliance.api_proxy()}


def _verify():
    return appliance.cert()


def _form_request_url(*, vault: vaults.Vault, secret_fragments: List[Optional[str]]) -> str:
    request_url = urljoin(
        vault.url, "secrets/{fragments}?api-version=7.1".format(fragments="/".join(list(filter(None, secret_fragments)))),
    )
    logger.debug(f"request url: {request_url}")
    return request_url


def _exception_handler(response_json: dict, status_code: int):
    logger.debug(f"handling exception for response: {response_json}")
    response_error = response_json.get("error")

    def _log_and_raise(exception, message=None):
        logger.error(f"status code: {status_code}")
        logger.error(f"response: {response_json}")
        ui.output_error(message or str(exception))
        raise exception

    if not response_error or not response_error.get("code"):
        # There should be a parent dictionary containing a single "error" key containing a "code" key. Fail early if neither exist.
        # https://docs.microsoft.com/en-us/rest/api/keyvault/getsecret/getsecret#keyvaulterror
        _log_and_raise(
            exceptions.UnhandledRequestError(f"Key Vault request failed with unhandled status code {status_code}."),
            "Azure Key Vault request raised an unhandled error. Please view the logs for more information.",
        )

    if status_code == 400:
        _log_and_raise(
            exceptions.BadRequest(
                "Key Vault request failed with status code 400. One or more of the supplied parameters may be invalid."
            )
        )
    elif status_code == 403:
        error = "Please view the logs for more information."
        if error_message := response_error.get("message"):
            if "not allowed on a disabled secret" in error_message:
                error = "The secret is disabled."
            elif "does not have secrets set permission" in error_message:
                error = "API token does not have permission to set the secret."
        exception = exceptions.PermissionDenied(f"Could not perform the Azure Key Vault operation. {error}")
        _log_and_raise(exception)
    elif status_code == 404:
        _log_and_raise(exceptions.SecretNotFound("The requested secret could not be found in the Azure Key Vault."))
    else:
        _log_and_raise(
            exceptions.UnhandledRequestError(
                f"Key Vault request failed with unhandled status code {status_code}. "
                f"Error message: '{response_error.get('code') or ''}'."
            )
        )


def _process_response(response: requests.Response, valid_status_codes: Tuple[int] = (200,)) -> Secret:
    # Even responses with errors should be valid JSON, so fail early if they're not.
    try:
        response_json = response.json()
    except json.JSONDecodeError as e:
        logger.exception(e)
        logger.error(response.text)
        ui.output_error("The response from Azure Key Vault was not valid JSON. Please view the logs for more information.")
        raise exceptions.BadResponseFormat("Failed to decode the response body to JSON.")

    if response.status_code in valid_status_codes:
        logger.debug(
            f"request was successful as status code {response.status_code} is in valid status codes {valid_status_codes}"
        )
        return response_json

    return _exception_handler(response_json, response.status_code)


def _get_key_vault(reason: str, vault_name: Optional[str] = None) -> vaults.Vault:
    if key_vaults := vaults._vaults_by_driver(driver_name, reason):
        if vault_name:
            try:
                return next(iter([v for v in key_vaults if v.name == vault_name]))
            except StopIteration:
                message = f"No Azure Key Vaults found in PPA with the name '{vault_name}'."
                ui.output_error(message)
                raise exceptions.NoKeyVaultFound(message)
        elif all([not vault_name, len(key_vaults) > 1]):
            ui.output_error("Multiple Azure Key Vaults found in PPA. Please select one from the table.")
            return key_vaults[
                ui.input_choice("Select a Key vault", header=["Name", "Address"], data=[[kv.name, kv.url] for kv in key_vaults])
            ]
        return next(iter(key_vaults))
    raise exceptions.NoKeyVaultFound("No Azure Key Vaults found in PPA.")


@appliance.min_ppa_version("2.7.0")
def _read_secret(*, name: str, reason: str, version: Optional[str] = None, key_vault: Optional[str] = None) -> Secret:
    # GET https://myvault.vault.azure.net/secrets/mysecretname/4387e9f3d6e14c459867679a90fd0f79?api-version=7.1
    return SecretData(
        **_process_response(
            requests.get(
                _form_request_url(vault=_get_key_vault(reason, vault_name=key_vault), secret_fragments=[name, version]),
                proxies=_proxies(),
                verify=_verify(),
            )
        )
    ).__dict__


@appliance.min_ppa_version("2.7.0")
def _set_secret(*, name: str, value: str, reason: str, key_vault: Optional[str] = None) -> Secret:
    # PUT https://myvault.vault.azure.net/secrets/crpsecret?api-version=7.1
    return SecretData(
        **_process_response(
            requests.put(
                _form_request_url(vault=_get_key_vault(reason, vault_name=key_vault), secret_fragments=[name]),
                proxies=_proxies(),
                verify=_verify(),
                json={"value": value},
            )
        )
    ).__dict__
