import datetime

from typing import Dict, TypedDict, Any, Optional

from datacaster.classes import CastDataClass


date_formatter = "%H:%M:%S %d/%m/%Y"


class SecretAttributes(TypedDict):
    created_at: str
    enabled: bool
    expiry: str
    updated_at: str


class SecretAttributesData(CastDataClass):
    __annotations__ = SecretAttributes.__dict__["__annotations__"]
    __class_config__ = {
        "rename_fields": {"created": "created_at", "updated": "updated_at", "exp": "expiry"},
        "always_cast": ["created_at", "updated_at", "expiry"],
        "cast_functions": {
            "fields": {
                "created_at": lambda value: datetime.datetime.fromtimestamp(value).strftime(date_formatter),
                "updated_at": lambda value: datetime.datetime.fromtimestamp(value).strftime(date_formatter),
                "expiry": lambda value: datetime.datetime.fromtimestamp(value).strftime(date_formatter),
            }
        },
    }


class Secret(TypedDict):
    """
- Contains Azure Key Vault secret data

!!! example

    ```yaml linenums="1"
    attributes:
      created_at: 11:09:18 23/12/2020
      enabled: true
      expiry: 11:09:18 23/12/2021
      updated_at: 11:09:18 23/12/2020
    secret_id: https://example.vault.azure.net/secrets/secret-name/secret-version
    tags:
      environment: production
    value: example_secret_value
    ```

!!! info "Secret Keys"

    __attributes__: A dictionary containing the following secret attributes: _created_at_, _enabled_, _expiry_, _updated_at_.

    __secret_id__: The ID of the secret.

    __tags__: A dictionary containing any tag names & values the secret has.

    __value__: The secret value.
    """

    attributes: SecretAttributes
    secret_id: str
    tags: Optional[Dict[str, Any]]
    value: str


class SecretData(CastDataClass):
    __annotations__ = Secret.__dict__["__annotations__"]
    __class_config__ = {
        "rename_fields": {"id": "secret_id"},
        "cast_functions": {"types": {SecretAttributes: lambda value: SecretAttributesData(**value).__dict__}},
    }
