"""Key Vault

This module contains actions for reading, creating, & updating secrets in an Azure Key Vault.

!!! danger "PPA Appliance & Vault Requirements"

    This plugin requires PPA version 2.7.0 or later.

    You must have an Azure Key Vault provisioned as a _Vault_ inside PPA to use this plugin.

!!! question "Azure API Version"

    This plugin supports Azure Key Vault API version 7.1.
"""

from typing import Optional

from ._key_vault import _read_secret, _set_secret, Secret


def read(
    name: str,
    version: Optional[str] = None,
    key_vault: Optional[str] = None,
    reason: Optional[str] = "Task is reading a secret from Azure Key Vault",
) -> Secret:
    """Read a secret from an Azure Key Vault.

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __name:__ the name of the secret

    - __version:__ optional secret version to read (defaults to the latest version)

    - __key_vault:__ the name of the Azure Key Vault in PPA (required if multiple Key Vaults are provisioned)

    - __reason:__ task interface audit message (defaults to _Task is reading a secret from Azure Key Vault_)

??? success "Output"

    A [Secret](../inputs_outputs/#secret) dictionary.

??? example

    Getting the _ansible_runner_key_ secret & saving it as a new variable called `secret`:

    ```YAML linenums="1"
    - azure_key_vault.secrets.read:
        name: ansible_runner_key
      save: secret
    ```
"""
    return _read_secret(name=name, key_vault=key_vault, version=version, reason=reason)


def set(
    name: str, value: str, key_vault: Optional[str] = None, reason: Optional[str] = "Task is setting a secret in Azure Key Vault",
) -> Secret:
    """Create a new _or_ update an existing secret in an Azure Key Vault.

If no secret exists with the supplied name, a new secret is created.

If a secret already exists with the supplied name, a new version of that secret will be created.

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __name:__ the name of the secret

    - __value:__ the value of the secret

    - __key_vault:__ the name of the Azure Key Vault in PPA (required if multiple Key Vaults are provisioned)

    - __reason:__ task interface audit message (defaults to _Task is setting a secret from Azure Key Vault_)

??? success "Output"

    A [Secret](../inputs_outputs/#secret) dictionary.

??? example

    Creating a secret called _ansible_runner_key_ & saving it as a new variable called `secret`:

    ```YAML linenums="1"
    {% raw %}
    - azure_key_vault.secrets.set:
        name: ansible_runner_key
        value: "{{ secret_value }}"
      save: secret
    {% endraw %}
    ```
"""
    return _set_secret(name=name, value=value, key_vault=key_vault, reason=reason)
